/*!
	\file	CAtlVersion.h
	\author	Abvent R&D Copyright (C) 2006
	\author	Guillaume Boulet-Gercourt
	\date	12 05 06
	\brief	Header for the CAtlVersion class
*/
#ifndef	CAtlVersion_h
#define	CAtlVersion_h

#ifndef	AvAtlTypes_h
	#include "CATL/AvAtlTypes.h"
#endif

#ifdef WIN32
	#define snprintf	_snprintf_s
#endif

namespace	AvAtl {

/// Enumeration of versions, used to determine file compatibility
enum EFileVersion
{
	/// the minimum version that can recognize the atl format (some data can be lost)
	kMinVersionMajor = 4,
	kMinVersionMinor = 1,
	kMinVersionRev = 0,

	/// the version that can read the file. No data will be lost
	kSafeVersionMajor = 4,
	kSafeVersionMinor = 1,
	kSafeVersionRev = 0,

	/// the current version of the file
	kCurrentVersionMajor = 4,
	kCurrentVersionMinor = 1,
	kCurrentVersionRev = 1
};


/*!
	\class CAtlVersion
	\author	Abvent R&D Copyright (C) 2006
	\author	Guillaume Boulet-Gercourt
	\date	12 05 06

	\brief	Tool class for atl version managment.
*/
class	CAtlVersion
{
public:
	/// Default constructor
	CAtlVersion() : mMajor(0), mMinor(0), mRevision(0) {}

	/// Constructor with major, minor, and revision version.
	CAtlVersion(int inMajor, int inMinor, int inRevision) : mMajor(inMajor), mMinor(inMinor), mRevision(inRevision) {}

	/// get major
	int	GetMajor() const { return mMajor; }
	/// set major
	void SetMajor(int inMajor) { mMajor = inMajor; }

	/// get minor
	int	GetMinor() const { return mMinor; }
	/// set minor
	void SetMinor(int inMinor) { mMinor = inMinor; }

	/// get revision
	int	GetRevision() const { return mRevision; }
	/// set revision
	void SetRevision(int inRevision) { mRevision = inRevision; }

	/// calculate major, minor, and revision from one number. (used for old file that have a version like "2,03" instead of "2.0.3"
	void		SetNumbers(double inVersions)
	{
		inVersions += 10e-6; // Add an epsilon to the versions.

		mMajor = (int)(inVersions);
		mMinor = (int)(inVersions * 10.0) % 10;
		mRevision = (int)(inVersions * 100.0) % 10;
	}

	/// set major, minor, and revision
	void		SetNumbers(int inMajor, int inMinor, int inRevision)
	{
		mMajor = inMajor;
		mMinor = inMinor;
		mRevision = inRevision;
	}

	/// return the version as a string : Major.Minor.Revision
	const char*		toString()
	{
		snprintf(mBuffer, sizeof(mBuffer) - 1, "%d.%d.%d", mMajor, mMinor, mRevision);
		mBuffer[sizeof(mBuffer) - 1] = 0;

		return mBuffer;
	}

	// comparaison between 2 atl versions
	bool		operator == (const CAtlVersion& inOther) const
	{
		return (mMajor == inOther.mMajor && mMinor == inOther.mMinor && mRevision == inOther.mRevision);
	}

	// comparaison between 2 atl versions
	bool		operator < (const CAtlVersion& inOther) const
	{
		if (mMajor < inOther.mMajor)
			return true;
		else if (mMajor > inOther.mMajor)
			return false;
		if (mMinor < inOther.mMinor)
			return true;
		else if (mMinor > inOther.mMinor)
			return false;
		if (mRevision < inOther.mRevision)
			return true;
		return false;
	}

	// comparaison between 2 atl versions
	bool		operator > (const CAtlVersion& inOther) const
	{
		if (mMajor > inOther.mMajor)
			return true;
		else if (mMajor < inOther.mMajor)
			return false;
		if (mMinor > inOther.mMinor)
			return true;
		else if (mMinor < inOther.mMinor)
			return false;
		if (mRevision > inOther.mRevision)
			return true;
		return false;
	}

	bool		operator <= (const CAtlVersion& inOther) const
	{
		if (mMajor > inOther.mMajor)
			return false;
		else if (mMajor < inOther.mMajor)
			return true;

		if (mMinor > inOther.mMinor)
			return false;
		else if (mMinor < inOther.mMinor)
			return true;

		if (mRevision > inOther.mRevision)
			return false;

		return true;
	}

	bool		operator >= (const CAtlVersion& inOther) const
	{
		if (mMajor < inOther.mMajor)
			return false;
		else if (mMajor > inOther.mMajor)
			return true;

		if (mMinor < inOther.mMinor)
			return false;
		else if (mMinor > inOther.mMinor)
			return true;

		if (mRevision < inOther.mRevision)
			return false;

		return true;
	}

	/// return true if initialised
	bool		IsValid() const { return mMajor != 0; }
private:

	int			mMajor;
	int			mMinor;
	int			mRevision;

	tUtf8Char	mBuffer[32];
};

/// Current ATL Version
#define		AtlCurrentVersion		CAtlVersion(kCurrentVersionMajor, kCurrentVersionMinor, kCurrentVersionRev)

/// Current ATL Safe Version
#define		AtlSafeVersion			CAtlVersion(kSafeVersionMajor, kSafeVersionMinor, kSafeVersionRev)

/// Current ATL Minimum Version
#define		AtlMinimumVersion		CAtlVersion(kMinVersionMajor, kMinVersionMinor, kMinVersionRev)

/// minimum version of ATL 1
#define		AtlV1MinVersion			CAtlVersion(1, 0, 0)

/// minimum version of ATL 2
#define		AtlV2MinVersion			CAtlVersion(2, 0, 0)

/// minimum version of ATL 3
#define		AtlV3MinVersion			CAtlVersion(3, 0, 0)

/// minimum version of ATL 4
#define		AtlV4MinVersion			CAtlVersion(4, 0, 0)

/// minimum version of ATL 4.1
#define		AtlV41MinVersion		CAtlVersion(4, 0, 2) /// version during beta test was 4.0.2, so I don't change it here....

/// minimum version of new ATL Animation Data
#define		AtlNewAnimationVersion	CAtlVersion(2, 0, 2)

/// version of artlantis 1.2
#define		AtlArtlantis12Version	CAtlVersion(2, 1, 0)

/// version of artlantis 2007 proto 1
#define		AtlArtlantis2007Proto1Version	CAtlVersion(2, 1, 13)

/// version of artlantis 2007 proto 2
#define		AtlArtlantis2007Proto2Version	CAtlVersion(2, 1, 14)

#define		AtlArtlantis2007Beta1Version	CAtlVersion(2, 1, 14)

#define		AtlArtlantis2007Beta2Version	CAtlVersion(2, 1, 14)

#define		AtlArtlantis2007Beta25Version	CAtlVersion(2, 1, 15)

#define		AtlArtlantisV3LayersProtoVersion	CAtlVersion(3, 0, 1)


#define		AtlArtlantis3NewRadioVersion CAtlVersion(3, 0, 0)

#define		AtlArtlantis3Alpha1Version CAtlVersion(3, 0, 1)

#define		AtlArtlantis3BetaVersion CAtlVersion(3, 0, 2)

#define		AtlArtlantisAOFVersion	AtlV2MinVersion


}	// namespace AvAtl

#endif	// defined CAtlVersion_h

