/*!
	\file	IAtlKeyFrame.h
	\date	04 08 05
	\author	Abvent R&D copyright (c) 2005
	\author	Guillaume Boulet-Gercourt
	Header for the IAtlKeyFrame interface.
*/
#ifndef		IAtlKeyFrame_h
# define	IAtlKeyFrame_h

// Basic class.
#ifndef AtlTypes_h
	#include "AvAtlTypes.h"
#endif
#ifndef TAtlObjectMgr_h
	#include "TAtlObjectMgr.h"
#endif
#ifndef CAtlVector_h
	#include "CAtlVector.h"
#endif
namespace	AvAtl {


/// List of key frames.
typedef TAtlObjectMgr<class IAtlKeyFrame>		AtlKeyFramesMgr;

/*!
	\interface	IAtlKeyFrame
	\date		04 08 05
	\author		Abvent R&D copyright (C) 2005
	\author		Guillaume Boulet-Gercourt
	\brief		Describes an key frame for Artlantis Studio
*/
class	IAtlKeyFrame
{
public:
	/// Put an XML representation of the key frame inside the father node.
	/** \retval the new node of the key frame.
		\retval NULL if an error occurs. */
	virtual XMLNodeRef				putInXML(XMLNodeRef inFatherNode) const = 0;
	/// Initialize the key frame with the XML.
	/** \retval true initialisation from XML succeed.
		\retval false an error occurs. */
	virtual bool					initWithXML(XMLNodeRef inFatherNode) = 0;

	/// eValueType : the type of the value of this key frame.
	enum	eValueType
	{
		kAtlVector = 0, // use "mCoord3D" in the union uValue
		kAtlColor = 1, // use "mColor" in the union uValue
		kDouble = 2, // use "mDouble" in the union uValue
		kBool = 3, // use "mBool" in the union uValue
		kInteger = 4, // use "mInteger" in the union uValue
		kDate = 5, // use "mInteger" in the union uValue
		kUUID = 6, // use "mUuid" in the union uValue
		kEnum = 7 // use "mInteger" in the union uValue.
	};

	/// eInterpolationType : the type of the interpolation of this key frame.
	enum	eInterpolationType
	{
		kLinear = 0, 
		kSquare = 1, 
		kOneMinusSquare = 2, 
		kSinus = 3, 
		kBezier = 4,
		kDateInterpolation = 5
	};
	/// get / set the value type
	virtual eValueType		getValueType() const = 0;
	virtual void			setValueType(eValueType inType) = 0;

	/// uValue : an union of all possibles types. The keyframe can describe a 3d vector, or a color, ...
	union	uValue
	{
		CAtlVector::ATLCoord	mCoord3D[3];
		unsigned char	mColor[4];
		double			mFloat;
		bool			mBool;
		int				mInteger;
		unsigned int	mUuid[4];

		/// Constructor - Initialize all datas to 0
		uValue() { mCoord3D[0] = 0; mCoord3D[1] = 0; mCoord3D[2] = 0;  }
	};

	/// get/set the value
	virtual const uValue&	getValue() const = 0;
	virtual void			setValue(const uValue& inValue) = 0;

	/// get/set the time of this keyframe. Time is the frame number
	virtual int				getTime() const = 0;
	virtual void			setTime(int inTime) = 0;

	virtual bool			getVisible() const = 0;
	virtual void			setVisible(bool inVisible) = 0;

	/// get/set the interpolation type of this keyframe.
	virtual eInterpolationType	getInterpolationType() const = 0;
	virtual void				setInterpolationType(eInterpolationType inType) = 0;

	/// get/set the before control point for this kf. (tangent)
	virtual const uValue&		getBeforeCtrlPoint() const = 0;
	virtual void				setBeforeCtrlPoint(const uValue& inValue) = 0;

	/// get/set the after control point for this kf. (tangent)
	virtual const uValue&		getAfterCtrlPoint() const = 0;
	virtual void				setAfterCtrlPoint(const uValue& inValue) = 0;

	/// get/set the flag to connect the tangents before and after the keyFrame
	virtual bool				getTangentLocked()const = 0;
	virtual void				setTangentLocked(bool inLocked) = 0;

protected:
	/** You must not create or delete objects of this class.
	    This is the responsibility of the CAtlMgr class. */
	virtual ~IAtlKeyFrame() {}
};

}

#endif
