/** \file
 * Header for the interface IAtlRenderingOpt
 *
 * \author Abvent R&D Copyright (c) 2000
 * \author Richard Young
 * \version 1.0
 * \date 03/07/2000
 */

#ifndef IAtlRenderingOpt_h
#define IAtlRenderingOpt_h


#ifndef AtlTypes_h
	#include "CATL/AvAtlTypes.h"
#endif
#ifndef EOutputType_h
	#include "CATL/EOutputType.h"
#endif
#ifndef EAntialias_h
	#include "CATL/EAntialias.h"
#endif


namespace  AvAtl {


/// Interface for view rendering ptions.
/** \interface IAtlRenderingOpt 
 *	Class that contains rendering options.
 *
 * \author Abvent R&D Copyright (c) 2000
 * \author Richard Young
 * \version 1.0
 * \date 03/07/2000
*/
class IAtlRenderingOpt
{
public:
	// Control the raytraced rendering.
	virtual bool				getRaytracingFlag() const = 0;
	virtual void				setRaytracingFlag(bool raytracing) = 0;

	// Control the shadow rendering.
	virtual bool				getShadowFlag() const = 0;
	virtual void				setShadowFlag(bool shadows) = 0;

	// Control the transparency rendering.
	virtual bool				getTransparencyFlag() const = 0;
	virtual void				setTransparencyFlag(bool transparency) = 0;

	// Control the production of alpha chanel (depth).
	virtual bool				getAlphaFlag() const = 0;
	virtual void				setAlphaFlag(bool alpha) = 0;

	// Control the number of reflections.
	// Value must be between 0 and 6.
	virtual int					getNbReflection() const = 0;
	virtual void				setNbReflection(int nbreflec) = 0;

	// Control the number of refractions.
	// Value must be between 0 and 16.
	virtual int					getNbRefraction() const = 0;
	virtual void				setNbRefraction(int nbrefrac) = 0;

	// Control the height of the image.
	// Value must be between 1 and 32000.
	virtual int					getHeight() const = 0;
	virtual void				setHeight(int height) = 0;

	// Control the width of the image.
	// Value must be between 1 and 32000.
	virtual int					getWidth() const = 0;
	virtual void				setWidth(int width) = 0;

	// The url of the file where the image will be stored.
	virtual tURL				getOutURL() const = 0;
	virtual void				setOutURL(tURL inURL) = 0;

	// The type of image.
	virtual EOutputType			getOutType() const = 0;
	virtual void				setOutType(EOutputType type) = 0;

	// Control the antialiasing level.
	virtual EAntialias			getAntialias() const = 0;
	virtual void				setAntialias(EAntialias antialias) = 0;

	// Control the rendering all option
	virtual bool				getRenderAllFlag() const = 0;
	virtual void				setRenderAllFlag(bool rendAll) = 0;

	// or Control the rendering first image
	virtual int					getRenderFirst() const = 0;
	virtual void				setRenderFirst(int firstImage) = 0;

	// and the rendering last image (for animation only)
	virtual int					getRenderLast() const = 0;
	virtual void				setRenderLast(int lastImage) = 0;

	// End of API 1

	/// Controls if this view must use view or the global options for rendering.
	virtual bool				getUseViewOpt() const = 0;
	/// Controls if this view must use view or the global options for rendering.
	virtual void				setUseViewOpt(bool inUseGlobal) = 0;

	/// Controls if the antialias is shown in the preview
	virtual bool				getPreviewAntialias() const = 0;
	/// Controls if the antialias is shown in the preview
	virtual void				setPreviewAntialias(bool inShowInPreview) = 0;

	/// Controls if the height and the width are linked.
	virtual bool				getHeightWidthLinked() const = 0;
	/// Controls if the height and the width are linked.
	virtual void				setHeightWidthLinked(bool inLinked) = 0;

	/// Get scale flag status
	/** For parallel views only */
	virtual bool				getParallelScaleFlag() const = 0;
	/// Set scale flag status
	/** For parallel views only */
	virtual void				setParallelScaleFlag(bool inFlag) = 0;

	/// Get scale (1/scale)
	/** For parallel views only */
	virtual int					getParallelScale() const = 0;
	/// Set scale (1/scale)
	/** For parallel views only */
	virtual void				setParallelScale(int inScale) = 0;

	/// Get image resolution (dpi)
	virtual int					getImageResolution() const = 0;
	/// Set image resolution (dpi)
	virtual void				setImageResolution(int inDPI) = 0;

	/// Get the image resolution.
	/**	For panorama only
	 0 -> 640 x 640
	 1 -> 1024 x 1024
	 2 -> 1600 x 1600
	 3 -> 2048 x 2048 */
	virtual int					getPanoramaResolution() const = 0;
	/// Set the image resolution.
	virtual void				setPanoramaResolution(int inResolution) = 0;

	/** RADIOSITY CONTROLS */
	/// Get the radiosity flag
    virtual bool				getRadiosityFlag() const = 0;
	/// Set the radiosity flag
	virtual void				setRadiosityFlag(bool inFlag) = 0;

	/// Get the radiosity factor
	virtual double				getRadiosityFactor() const = 0;
	/// Set the radiosity factor
	virtual void				setRadiosityFactor(double inFactor) = 0;

	/// Get the radiosity quality
	virtual double				getRadiosityQuality() const = 0;
    /// Set the radiosity quality
	virtual void				setRadiosityQuality(double inQuality) = 0;

	/// Get the contrast value
	virtual double				getContrast() const = 0;
    /// Set the contrast value
    /** Default is 1 and normal range is between 0.1 and 10. */
	virtual void				setContrast(double inContrast) = 0;

	/// Get the luminosity value
	virtual double				getLuminosity() const = 0;
    /// Set the luminosity value
    /** Default is 0 and normal range is between -0.5 and 0.5. */
	virtual void				setLuminosity(double inLuminosity) = 0;

	/// Get the long range [16, 96], default 96.0
	virtual double				getLongRange() const = 0;
	/// Set the long range
	virtual	void				setLongRange(double inLongRange) = 0;

    /// Get the short range [0.0, 32.0], default 4.0
	virtual double				getShortRange() const = 0;
    /// Set the short range [16, 96], default 96.0
	virtual	void				setShortRange(double inShortRange) = 0;

    /// Get the first bounce [1.0, 16.0], default 4.0
	virtual double				getFirstBounce() const = 0;
    /// Set the first bounce
	virtual	void				setFirstBounce(double inFirstBounce) = 0;

    /// Get the next bounces [1.0, 16.0], default 1.0
	virtual double				getNextBounces() const = 0;
    /// Set the next bounces
	virtual	void				setNextBounces(double inNextBounces) = 0;

    /// Get the attenuation [0.0, 1.0], default 0.3 (good for exterior)
	virtual double				getAttenuation() const = 0;
    /// Set the attenuation
	virtual	void				setAttenuation(double inAttenuation) = 0;

	/// Get the  color bleeding [0.0, 1.0], default 1.0 (good for exterior)
	virtual double				getColorBleeding() const = 0;
	/// Set the color bleeding
	virtual	void				setColorBleeding(double inColorBleeding) = 0;

	/// Get Additional XML data of rendering opts. For Consulting
	virtual ConstXMLNodeRef		getAdditionnalOpts() const = 0;
	/// Get Additional XML data of rendering opts. For Editing
	virtual XMLNodeRef			getAdditionnalOpts() = 0;

	/// Get the ISO value
	virtual unsigned int		getISO() const = 0;
	/// Set the ISO
	virtual void				setISO(unsigned int inISO) = 0;

	/// Get the shutter speed
	virtual unsigned int		getShutter() const = 0;
	/// Set the shutter speed
	virtual void				setShutter(unsigned int inShutter) = 0;

	/// Color Adjustement Behavior
	enum	eColorAdjustement
	{
		kUseAutolum = 0,
		kUseIsoShutters
	};
	virtual eColorAdjustement	getColorAdjustementBehavior() const = 0;
	virtual void				setColorAdjustementBehavior(eColorAdjustement inBehavior) = 0;

protected:
	// You must not create or delete objects of this class.
	// This is the responsability of the CAtlMgr class.
								~IAtlRenderingOpt() {}
};


} // namespace AvAtl

#endif
